from stats_collector import FRED_DB
from stats_collector.stats import Statistics
from stats_collector.stats.stat_registry import stat_registry


@stat_registry.register()
class DomainOwners(Statistics):
    sql = """
        WITH first_domainname_registrations AS (
            SELECT oregd.name, MIN(oregd.crdate) AS first_registration_date
              FROM object_registry oregd
             WHERE oregd.type = get_object_type_id('domain'::text)
             GROUP BY oregd.name
        )
        SELECT oregd.name AS domainname,
               fdr.first_registration_date,
               dh.exdate AS expiration_date,
               r.handle AS registrar_id,
               oregc.name AS subscriber_id,
               ch.country AS subscriber_country,
               ch.postalcode AS subscriber_psc
          FROM object_registry oregd
          JOIN object_history oh ON (oregd.id = oh.id)
          JOIN history h ON (h.id = oh.historyid)
          JOIN domain_history dh ON (h.id = dh.historyid)
          JOIN object_registry oregc ON (dh.registrant = oregc.id)
          JOIN contact_history ch ON (oregc.id = ch.id)
          JOIN history hc ON (hc.id = ch.historyid)
          JOIN registrar r ON (oh.clid = r.id)
          JOIN first_domainname_registrations fdr ON (oregd.name = fdr.name)
         WHERE oregd.type = get_object_type_id('domain'::text)
           AND oregd.erdate IS NULL
           AND oregc.type = get_object_type_id('contact'::text)
           AND h.valid_from < %(midnight)s
           AND (h.valid_to >= %(midnight)s OR h.valid_to IS NULL)
           AND hc.valid_from < %(midnight)s
           AND (hc.valid_to >= %(midnight)s OR hc.valid_to IS NULL)
         ORDER BY oregd.name
    """
    name = 'domain_owners'
    stat_db = FRED_DB
