from stats_collector import FRED_DB, LOG_DB
from stats_collector.stats import Statistics
from stats_collector.stats.stat_registry import stat_registry

from ..groups import FRED_DAILY_GROUP


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainCreate(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'BillingFailure' THEN 'billing_failure'
                WHEN rc.name = 'ObjectExists' THEN 'object_exists'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND LOWER(rt.name) = LOWER('DomainCreate')
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('DomainCreate')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainUpdate(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'ObjectDoesNotExist' THEN 'object_does_not_exist'
                WHEN rc.name = 'AuthorizationError' THEN 'authorization_error'
                WHEN rc.name = 'ObjectStatusProhibitsOperation' THEN 'object_status_prohibits_operation'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND LOWER(rt.name) = LOWER('DomainUpdate')
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('DomainUpdate')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainRenew(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'BillingFailure' THEN 'billing_failure'
                WHEN rc.name = 'AuthorizationError' THEN 'authorization_error'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND LOWER(rt.name) = LOWER('DomainRenew')
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('DomainRenew')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainTransfer(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'ObjectDoesNotExist' THEN 'object_does_not_exist'
                WHEN rc.name = 'AuthorizationError' THEN 'authorization_error'
                WHEN rc.name = 'ObjectStatusProhibitsOperation' THEN 'object_status_prohibits_operation'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND LOWER(rt.name) = LOWER('DomainTransfer')
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('DomainTransfer')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainCheck(Statistics):
    sql = '''
        SELECT
            rt.name,
            'success',
            COUNT(*)
          FROM
            request_type rt
            JOIN request r ON r.request_type_id = rt.id
            JOIN result_code rc ON rc.id = r.result_code_id
          WHERE
            %(log_req_cond_epp)s
            AND LOWER(rt.name) = LOWER('DomainCheck')
            AND LOWER(rc.name) = LOWER('CommandCompletedSuccessfully')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainInfo(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'ObjectDoesNotExist' THEN 'object_does_not_exist'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND LOWER(rt.name) = LOWER('DomainInfo')
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('DomainInfo')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainSendAuthInfo(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'ObjectDoesNotExist' THEN 'object_does_not_exist'
                WHEN rc.name = 'ObjectStatusProhibitsOperation' THEN 'object_status_prohibits_operation'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND LOWER(rt.name) = LOWER('DomainSendAuthInfo')
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('DomainSendAuthInfo')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationbyResultDomainDelete(Statistics):
    sql = '''
        SELECT
            'DomainDelete', 'success_registry', COUNT(*)
          FROM
              object_registry O
              JOIN domain_history D ON (D.historyid = O.crhistoryid)
              JOIN zone Z ON (D.zone = Z.id)
              JOIN object_state S ON (O.id = S.object_id)
          WHERE
              erdate BETWEEN '%(date)s'::timestamp AND %(midnight)s
              -- deleteCandidate which was not canceled (e.g. unblocking after law suits)
              AND S.state_id = 17 AND valid_to IS NULL
              AND O.type = 3
              AND Z.fqdn = 'cz'
        UNION
        SELECT
            'DomainDelete', 'success_registrars', COUNT(*)
          FROM
            object_registry oreg
            JOIN domain_history dh ON dh.historyid = oreg.crhistoryid
            JOIN zone z ON z.id = dh.zone
            LEFT JOIN object_state os ON os.object_id = oreg.id AND os.state_id = 17
          WHERE
            erdate BETWEEN '%(date)s'::timestamp AND %(midnight)s
            AND oreg.erdate IS NOT NULL
            AND oreg.type = 3
            AND z.fqdn = 'cz'
            AND os.id IS NULL
    '''
    name = 'request_type_by_result_count'
    stat_db = FRED_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationbyResultContact(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                ELSE 'fail'
            END,
            COALESCE(COUNT(foo.r_id), 0)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND (
                        LOWER(rt.name) = LOWER('ContactCreate')
                        OR LOWER(rt.name) = LOWER('ContactUpdate')
                        OR LOWER(rt.name) = LOWER('ContactTransfer')
                        OR LOWER(rt.name) = LOWER('ContactCheck')
                        OR LOWER(rt.name) = LOWER('ContactInfo')
                        OR LOWER(rt.name) = LOWER('ContactSendAuthInfo')
                        OR LOWER(rt.name) = LOWER('ContactDelete')
                    )
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('ContactCreate')
                OR LOWER(rt.name) = LOWER('ContactUpdate')
                OR LOWER(rt.name) = LOWER('ContactTransfer')
                OR LOWER(rt.name) = LOWER('ContactCheck')
                OR LOWER(rt.name) = LOWER('ContactInfo')
                OR LOWER(rt.name) = LOWER('ContactSendAuthInfo')
                OR LOWER(rt.name) = LOWER('ContactDelete')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationbyResultNSSet(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                ELSE 'fail'
            END,
            COALESCE(COUNT(foo.r_id), 0)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND (
                        LOWER(rt.name) = LOWER('NSsetCreate')
                        OR LOWER(rt.name) = LOWER('NSsetUpdate')
                        OR LOWER(rt.name) = LOWER('NSsetTransfer')
                        OR LOWER(rt.name) = LOWER('NSsetCheck')
                        OR LOWER(rt.name) = LOWER('NSsetInfo')
                        OR LOWER(rt.name) = LOWER('NSsetSendAuthInfo')
                        OR LOWER(rt.name) = LOWER('NSsetDelete')
                    )
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('NSsetCreate')
                OR LOWER(rt.name) = LOWER('NSsetUpdate')
                OR LOWER(rt.name) = LOWER('NSsetTransfer')
                OR LOWER(rt.name) = LOWER('NSsetCheck')
                OR LOWER(rt.name) = LOWER('NSsetInfo')
                OR LOWER(rt.name) = LOWER('NSsetSendAuthInfo')
                OR LOWER(rt.name) = LOWER('NSsetDelete')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationbyResultKeySet(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                ELSE 'fail'
            END,
            COALESCE(COUNT(foo.r_id), 0)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND (
                        LOWER(rt.name) = LOWER('KeysetCreate')
                        OR LOWER(rt.name) = LOWER('KeysetUpdate')
                        OR LOWER(rt.name) = LOWER('KeysetTransfer')
                        OR LOWER(rt.name) = LOWER('KeysetCheck')
                        OR LOWER(rt.name) = LOWER('KeysetInfo')
                        OR LOWER(rt.name) = LOWER('KeySetSendAuthInfo')
                        OR LOWER(rt.name) = LOWER('KeysetDelete')
                    )
                ) foo ON LOWER(foo.rt_name) = LOWER(rt.name) AND LOWER(foo.rc_name) = LOWER(rc.name)
          WHERE
                LOWER(rt.name) = LOWER('KeysetCreate')
                OR LOWER(rt.name) = LOWER('KeysetUpdate')
                OR LOWER(rt.name) = LOWER('KeysetTransfer')
                OR LOWER(rt.name) = LOWER('KeysetCheck')
                OR LOWER(rt.name) = LOWER('KeysetInfo')
                OR LOWER(rt.name) = LOWER('KeySetSendAuthInfo')
                OR LOWER(rt.name) = LOWER('KeysetDelete')
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB
