"""Tests for data_collector module."""
import csv
import os
import tempfile
from datetime import date
from unittest import TestCase

from mock import patch

from stats_collector.data_collector import DataCollector


class TestDataCollector(TestCase):
    """Unittests for DataCollector."""

    def test_save_data(self):
        data = {'crew': [['David', 'Lister'], ['Arnold', 'Rimmer']]}
        with tempfile.TemporaryDirectory() as temp_dir:
            with patch('stats_collector.config.STAT_DIR', temp_dir):
                collector = DataCollector()
                collector.run_date = date(2019, 11, 16)

                collector.save_data(data)

            with open(os.path.join(temp_dir, '20191116-crew.csv')) as csv_file:
                csv_reader = csv.reader(csv_file, delimiter='|')
                self.assertEqual(list(csv_reader), data['crew'])

    @patch('stats_collector.data_collector.DataCollector.get_data')
    def test_get_and_save_data_no_data(self, get_data_mock):
        get_data_mock.return_value = None
        rc = DataCollector().get_and_save_data(None)
        self.assertEqual(rc, 0)
